@@ -13,19 +13,24 @@
 """SQLAlchemy storage backend."""
 
 from __future__ import absolute_import
+import copy
 import datetime
 import os.path
 
 from alembic import command
 from alembic import config
 from alembic import migration
-from oslo_db import exception
 from oslo_db.sqlalchemy import session as db_session
+from oslo_db.sqlalchemy import utils as oslo_sql_utils
 from oslo_log import log
 from oslo_utils import timeutils
 import six
+from sqlalchemy import asc
 from sqlalchemy import desc
+from sqlalchemy import func
+from sqlalchemy.orm import exc
 
+import aodh
 from aodh.i18n import _LI
 from aodh import storage
 from aodh.storage import base
@@ -90,15 +95,8 @@ class Connection(base.Connection):
             ctxt = migration.MigrationContext.configure(engine.connect())
             current_version = ctxt.get_current_revision()
             if current_version is None:
-                try:
-                    models.Base.metadata.create_all(engine, checkfirst=False)
-                except exception.DBError:
-                    # Assume tables exist from Ceilometer, take control
-                    # FIXME(jd) Remove in Ocata
-                    command.stamp(cfg, "12fe8fac9fe4")
-                    command.upgrade(cfg, "head")
-                else:
-                    command.stamp(cfg, "head")
+                models.Base.metadata.create_all(engine, checkfirst=False)
+                command.stamp(cfg, "head")
             else:
                 command.upgrade(cfg, "head")
 
@@ -151,9 +149,44 @@ class Connection(base.Connection):
     def _retrieve_alarms(self, query):
         return (self._row_to_alarm_model(x) for x in query.all())
 
+    @staticmethod
+    def _get_pagination_query(session, query, pagination, api_model, model):
+        if not pagination.get('sort'):
+            pagination['sort'] = api_model.DEFAULT_SORT
+        marker = None
+        if pagination.get('marker'):
+            key_attr = getattr(model, api_model.PRIMARY_KEY)
+            marker_query = copy.copy(query)
+            marker_query = marker_query.filter(
+                key_attr == pagination['marker'])
+            try:
+                marker = marker_query.limit(1).one()
+            except exc.NoResultFound:
+                raise storage.InvalidMarker(
+                    'Marker %s not found.' % pagination['marker'])
+        limit = pagination.get('limit')
+        # we sort by "severity" by its semantic than its alphabetical
+        # order when "severity" specified in sorts.
+        for sort_key, sort_dir in pagination['sort'][::-1]:
+            if sort_key == 'severity':
+                engine = session.connection()
+                if engine.dialect.name != "mysql":
+                    raise aodh.NotImplementedError
+                sort_dir_func = {'asc': asc, 'desc': desc}[sort_dir]
+                query = query.order_by(sort_dir_func(
+                    func.field(getattr(model, sort_key), 'low',
+                               'moderate', 'critical')))
+                pagination['sort'].remove((sort_key, sort_dir))
+
+        sort_keys = [s[0] for s in pagination['sort']]
+        sort_dirs = [s[1] for s in pagination['sort']]
+        return oslo_sql_utils.paginate_query(
+            query, model, limit, sort_keys, sort_dirs=sort_dirs, marker=marker)
+
     def get_alarms(self, name=None, user=None, state=None, meter=None,
                    project=None, enabled=None, alarm_id=None,
-                   alarm_type=None, severity=None, exclude=None):
+                   alarm_type=None, severity=None, exclude=None,
+                   pagination=None):
         """Yields a lists of alarms that match filters.
 
         :param name: Optional name for alarm.
@@ -166,8 +199,10 @@ class Connection(base.Connection):
         :param alarm_type: Optional alarm type.
         :param severity: Optional alarm severity.
         :param exclude: Optional dict for inequality constraint.
+        :param pagination: Pagination query parameters.
         """
 
+        pagination = pagination or {}
         session = self._engine_facade.get_session()
         query = session.query(models.Alarm)
         if name is not None:
@@ -190,7 +225,8 @@ class Connection(base.Connection):
             for key, value in six.iteritems(exclude):
                 query = query.filter(getattr(models.Alarm, key) != value)
 
-        query = query.order_by(desc(models.Alarm.timestamp))
+        query = self._get_pagination_query(
+            session, query, pagination, alarm_api_models.Alarm, models.Alarm)
         alarms = self._retrieve_alarms(query)
 
         # TODO(cmart): improve this by using sqlalchemy.func factory
@@ -221,10 +257,12 @@ class Connection(base.Connection):
         """
         session = self._engine_facade.get_session()
         with session.begin():
-            alarm_row = session.merge(models.Alarm(alarm_id=alarm.alarm_id))
-            alarm_row.update(alarm.as_dict())
-
-        return self._row_to_alarm_model(alarm_row)
+            count = session.query(models.Alarm).filter(
+                models.Alarm.alarm_id == alarm.alarm_id).update(
+                    alarm.as_dict())
+            if not count:
+                raise storage.AlarmNotFound(alarm.alarm_id)
+        return alarm
 
     def delete_alarm(self, alarm_id):
         """Delete an alarm and its history data.
@@ -268,7 +306,7 @@ class Connection(base.Connection):
                           user=None, project=None, alarm_type=None,
                           severity=None, start_timestamp=None,
                           start_timestamp_op=None, end_timestamp=None,
-                          end_timestamp_op=None):
+                          end_timestamp_op=None, pagination=None):
         """Yields list of AlarmChanges describing alarm history
 
         Changes are always sorted in reverse order of occurrence, given
@@ -292,7 +330,9 @@ class Connection(base.Connection):
         :param start_timestamp_op: Optional timestamp start range operation
         :param end_timestamp: Optional modified timestamp end range
         :param end_timestamp_op: Optional timestamp end range operation
+        :param pagination: Pagination query parameters.
         """
+        pagination = pagination or {}
         session = self._engine_facade.get_session()
         query = session.query(models.AlarmChange)
         query = query.filter(models.AlarmChange.alarm_id == alarm_id)
@@ -323,7 +363,9 @@ class Connection(base.Connection):
                 query = query.filter(
                     models.AlarmChange.timestamp < end_timestamp)
 
-        query = query.order_by(desc(models.AlarmChange.timestamp))
+        query = self._get_pagination_query(
+            session, query, pagination, alarm_api_models.AlarmChange,
+            models.AlarmChange)
         return self._retrieve_alarm_history(query)
 
     def record_alarm_change(self, alarm_change):
