@@ -13,11 +13,10 @@
 """Tests for aodh/evaluator/composite.py
 """
 
-import uuid
-
 from ceilometerclient.v2 import statistics
 import mock
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 from oslotest import mockpatch
 import six
 from six import moves
@@ -169,7 +168,7 @@ class CompositeTest(BaseCompositeEvaluate):
                          enabled=True,
                          user_id='fake_user',
                          project_id='fake_project',
-                         alarm_id=str(uuid.uuid4()),
+                         alarm_id=uuidutils.generate_uuid(),
                          state='insufficient data',
                          state_timestamp=constants.MIN_DATETIME,
                          timestamp=constants.MIN_DATETIME,
@@ -197,7 +196,7 @@ class CompositeTest(BaseCompositeEvaluate):
                          ok_actions=[],
                          alarm_actions=[],
                          repeat_actions=False,
-                         alarm_id=str(uuid.uuid4()),
+                         alarm_id=uuidutils.generate_uuid(),
                          time_constraints=[],
                          rule={
                              "or": [self.sub_rule1, self.sub_rule2,
@@ -218,7 +217,7 @@ class CompositeTest(BaseCompositeEvaluate):
                          ok_actions=[],
                          alarm_actions=[],
                          repeat_actions=False,
-                         alarm_id=str(uuid.uuid4()),
+                         alarm_id=uuidutils.generate_uuid(),
                          time_constraints=[],
                          rule={
                              "and": [self.sub_rule1, self.sub_rule2,
@@ -232,7 +231,7 @@ class CompositeTest(BaseCompositeEvaluate):
                          enabled=True,
                          user_id='fake_user',
                          project_id='fake_project',
-                         alarm_id=str(uuid.uuid4()),
+                         alarm_id=uuidutils.generate_uuid(),
                          state='insufficient data',
                          state_timestamp=constants.MIN_DATETIME,
                          timestamp=constants.MIN_DATETIME,
@@ -463,7 +462,7 @@ class OtherCompositeTest(BaseCompositeEvaluate):
                          ok_actions=['log://'],
                          alarm_actions=['log://'],
                          repeat_actions=False,
-                         alarm_id=str(uuid.uuid4()),
+                         alarm_id=uuidutils.generate_uuid(),
                          time_constraints=[],
                          rule={
                              "or": [self.sub_rule1, self.sub_rule2]
