@@ -17,12 +17,12 @@
 import copy
 import datetime
 import json
-import uuid
 
 from ceilometerclient import exc
 from ceilometerclient.v2 import statistics
 import mock
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import pytz
 from six import moves
 
@@ -44,7 +44,7 @@ class TestEvaluate(base.TestEvaluatorBase):
                          enabled=True,
                          user_id='foobar',
                          project_id='snafu',
-                         alarm_id=str(uuid.uuid4()),
+                         alarm_id=uuidutils.generate_uuid(),
                          state='insufficient data',
                          state_timestamp=constants.MIN_DATETIME,
                          timestamp=constants.MIN_DATETIME,
@@ -81,7 +81,7 @@ class TestEvaluate(base.TestEvaluatorBase):
                          ok_actions=[],
                          alarm_actions=[],
                          repeat_actions=False,
-                         alarm_id=str(uuid.uuid4()),
+                         alarm_id=uuidutils.generate_uuid(),
                          time_constraints=[],
                          rule=dict(
                              comparison_operator='le',
@@ -193,6 +193,56 @@ class TestEvaluate(base.TestEvaluatorBase):
                     in zip(self.alarms, reasons, reason_datas)]
         self.assertEqual(expected, self.notifier.notify.call_args_list)
 
+    @mock.patch.object(timeutils, 'utcnow')
+    def test_lag_configuration(self, mock_utcnow):
+        mock_utcnow.return_value = datetime.datetime(2012, 7, 2, 10, 45)
+        self.api_client.statistics.list.side_effect = []
+
+        self._set_all_alarms('ok')
+        self._evaluate_all_alarms()
+        self._set_all_alarms('ok')
+        self.conf.set_override("additional_ingestion_lag", 42)
+        self._evaluate_all_alarms()
+
+        self.assertEqual([
+            mock.call(
+                meter_name='cpu_util', period=60,
+                q=[{'value': 'cpu_util', 'op': 'eq', 'field': 'meter'},
+                   {'value': 'my_instance', 'op': 'eq',
+                    'field': 'resource_id'},
+                   {'value': '2012-07-02T10:45:00', 'op': 'le',
+                    'field': 'timestamp'},
+                   {'value': '2012-07-02T10:39:00', 'op': 'ge',
+                    'field': 'timestamp'}]),
+            mock.call(
+                meter_name='cpu_util', period=300,
+                q=[{'value': 'cpu_util', 'op': 'eq', 'field': 'meter'},
+                   {'value': 'my_group', 'op': 'eq',
+                    'field': 'metadata.user_metadata.AS'},
+                   {'value': '2012-07-02T10:45:00', 'op': 'le',
+                    'field': 'timestamp'},
+                   {'value': '2012-07-02T10:20:00', 'op': 'ge',
+                    'field': 'timestamp'}]),
+            mock.call(
+                meter_name='cpu_util', period=60,
+                q=[{'value': 'cpu_util', 'op': 'eq', 'field': 'meter'},
+                   {'value': 'my_instance', 'op': 'eq',
+                    'field': 'resource_id'},
+                   {'value': '2012-07-02T10:45:00', 'op': 'le',
+                    'field': 'timestamp'},
+                   {'value': '2012-07-02T10:38:18', 'op': 'ge',
+                    'field': 'timestamp'}]),
+            mock.call(
+                meter_name='cpu_util', period=300,
+                q=[{'value': 'cpu_util', 'op': 'eq', 'field': 'meter'},
+                   {'value': 'my_group', 'op': 'eq',
+                    'field': 'metadata.user_metadata.AS'},
+                   {'value': '2012-07-02T10:45:00', 'op': 'le',
+                    'field': 'timestamp'},
+                   {'value': '2012-07-02T10:19:18', 'op': 'ge',
+                    'field': 'timestamp'}])],
+            self.api_client.statistics.list.mock_calls)
+
     def test_simple_alarm_clear(self):
         self._set_all_alarms('alarm')
         avgs = [self._get_stat('avg', self.alarms[0].rule['threshold'] - v)
@@ -218,12 +268,18 @@ class TestEvaluate(base.TestEvaluatorBase):
 
     def _construct_payloads(self):
         payloads = []
+        reasons = ["Transition to alarm due to 5 samples outside threshold, "
+                   "most recent: 85.0",
+                   "Transition to alarm due to 4 samples outside threshold, "
+                   "most recent: 7.0"]
         for alarm in self.alarms:
+            num = self.alarms.index(alarm)
             type = models.AlarmChange.STATE_TRANSITION
-            detail = json.dumps({'state': alarm.state})
+            detail = json.dumps({'state': alarm.state,
+                                 'transition_reason': reasons[num]})
             on_behalf_of = alarm.project_id
             payload = dict(
-                event_id='fake_event_id_%s' % self.alarms.index(alarm),
+                event_id='fake_event_id_%s' % num,
                 alarm_id=alarm.alarm_id,
                 type=type,
                 detail=detail,
@@ -234,7 +290,7 @@ class TestEvaluate(base.TestEvaluatorBase):
             payloads.append(payload)
         return payloads
 
-    @mock.patch.object(uuid, 'uuid4')
+    @mock.patch.object(uuidutils, 'generate_uuid')
     @mock.patch.object(timeutils, 'utcnow')
     @mock.patch.object(messaging, 'get_notifier')
     def test_alarm_change_record(self, get_notifier, utcnow, mock_uuid):
